function [output extraOutput] = d_SVM(set1input,set2input,varargin)

% train, predict, compare to null
%
% input of each group is a matrix in form: features (x features...) x
% instances.  Instances must be the last dimension.
%
%
% on Wes's data this is:
% /Volumes/Hoffman2/mscohen/data/kerr/PET/PETData_LTLEvsNES.mat
%
% Unshuffled     NullDists ---->>>
% testComb1Perm1 testComb1Perm2 ....
% testComb2Perm1 testComb2Perm2
% testComb3Perm1 testComb3Perm2
% ....
%
%
%
% set1 has label 0; set2 has label 1


p = inputParser;
p.addRequired('set1input');
p.addRequired('set2input');

% Params Affecting Data Accuracy
p.addParamValue('solver',1,@isnumeric);
p.addParamValue('eqTrainAmmounts',false,@(x)islogical(x) || isnumeric(x));
p.addParamValue('testAmmount',1,@isnumeric);
p.addParamValue('doPCA',false,@(x)islogical(x) || isnumeric(x));

p.addParamValue('nNullPerms',2,@isnumeric);
p.addParamValue('isQuiet',true,@(x)islogical(x) || isnumeric(x));

p.addParamValue('baseDir','',@ischar);
p.addParamValue('recursiveRegExp',false,@(x)islogical(x) || isnumeric(x));

p.addParamValue('verbose',true,@(x)islogical(x) || isnumeric(x));

p.parse(set1input,set2input,varargin{:});
vars = fields(p.Results);
for i = 1:numel(vars)
    eval([vars{i} '=p.Results.' vars{i} ';'])
end

if isempty(baseDir) %#ok<*NODEF>
    baseDir = pwd;
end

set1Label = -1;
set2Label = 1;

[output.tMat output.acc output.accNull output.accP output.weightsP] = deal(NaN); %output parameter
output.s1.imageFiles = {};
output.s2.imageFiles = {};

%% Execute RegExps
for setN = 1:2
    setNchar = num2str(setN);
    eval(['setInput = set' setNchar 'input;'])
    
    %% Find Matching Images
    if ischar(setInput)
        setInputChar = setInput;
        
        % char to cell w/ filepaths as elements
        setInput = regexpdir(baseDir,setInput,recursiveRegExp); %note: directory-recursive
        imageFiles{setN} = setInput;
        eval(['output.s' setNchar '.imageFiles = setInput;'])
        
        if isempty(setInput)
            disp(['no files matched ' setInputChar ' in ' baseDir])
            return
        end
    end
    
    %% Load Images
    if iscell(setInput)
        setFilepaths = setInput;
        
        vInfo = spm_vol(setFilepaths);
        output.tMat = vInfo{1}.mat; % images need to all have same .mat so doesn't matter which one we pick so pick #1
        
        if numel(vInfo) <= 1
            disp(['<= 1 files in either set1/set2'])
            return
        end
        
        if verbose; disp('loading images...'); end
        setData = spm_read_vols([vInfo{:}]);

        if verbose; disp('done'); end

        eval(['set' setNchar 'input = setData;'])
        
    end
end

%% Reshape Input Data from F(xFxF...)xI to IxF to feed into train/predict

sSet1 = size(set1input);
set1featureDims = sSet1(1:end-1);
nF1 = prod(set1featureDims);
nI1 = sSet1(end);
set1Data = reshape(set1input,[nF1 nI1])';

sSet2 = size(set2input);
set2featureDims = sSet2(1:end-1);
nF2 = prod(set2featureDims);
nI2 = sSet2(end);
set2Data = reshape(set2input,[nF2 nI2])';

try
    if ~all(set1featureDims == set2featureDims)
        disp(sprintf('\n%d instances in set1 and %d instances in set2',nI1,nI2))
        disp(sprintf('\n%d features in set1 and %d features in set2',nF1,nF2))
        error('feature dims are not the same size across datasets')
    else
        featureDims = set1featureDims;
    end
catch
    error('feature dims are not the same size across datasets')
    disp(sprintf('\n%d instances in set1 and %d instances in set2',nI1,nI2))
    disp(sprintf('\n%d features in set1 and %d features in set2',nF1,nF2))
end

if verbose
    disp(sprintf('\n%d instances in set1 and %d instances in set2',nI1,nI2))
    disp(sprintf('\n%d features in set1 and %d features in set2',nF1,nF2))
end

if nI1 <= 1
    disp('1 or less instance(s) in set1');
    return
elseif nI2 <= 1
    disp('1 or less instance(s) in set2');
    return
end

%% Creat train/predict options
trainOpts = ['-q'*isQuiet ' -s ' num2str(solver)];
predictOpt = ['-q'*isQuiet ''];

dataRaw = cat(1,set1Data,set2Data);
labels = cat(1, set1Label*ones(nI1,1), set2Label*ones(nI2,1));


%% Remove any NaN features from data
noNanVox = var(dataRaw) > 0;
data = dataRaw(:,noNanVox);
nFs = size(data,2);
nIs = nI1 + nI2;
if verbose
    disp([num2str(nFs) ' features left after removing NaNs and low variance'])
end

if nFs == 0
    error('no remaining features after removing NaNs')
end

if doPCA
    data = pca(data)';
end
data = sparse(data);


%% Create Combinations to test on
instanceInds = 1:nIs;

if testAmmount < 1 % allow percentages
    testQuantity = ceil(testAmmount * size(dataRaw,1))
else
    testQuantity = testAmmount;
end

if eqTrainAmmounts % only works if same number of input instances for now
%     if nI1 ~= nI2
%         error('this option really only makes sense if there are == numbers of instances in both cats')
%     end

    labelVals = unique(labels);
    s1inds = find(labels == labelVals(1));
    s2inds = find(labels == labelVals(2));
    
    testCombs1 = nchoosek(s1inds,testQuantity);
    testCombs2 = nchoosek(s2inds,testQuantity);
    
    testCombs = combvec(testCombs1',testCombs2')';
else
    testCombs = nchoosek(instanceInds,testQuantity); % choose which to test
end

[nTestCombs nInstancesPerTest] = size(testCombs);
nTestPerms = 1 + nNullPerms; % add 1 here to include the (1st) unshuffled version

%% UPDATE THIS TO REFLECT SIGNIFICANCE TESTING
if verbose
    disp([num2str(nTestCombs) ' instance combinations; ' num2str(nTestPerms)...
        ' label permutations each = ' num2str(nTestPerms*nTestCombs) ' iterations'])
end


%% Run Accuracy & Significance Permutations
accLabelByCombByPerm = nan(nTestCombs, nInstancesPerTest, nTestPerms); %prealloc for speed
allTestLabels = nan(nTestCombs, nInstancesPerTest, nTestPerms); %prealloc for speed
allPreds = nan(nTestCombs, nInstancesPerTest, nTestPerms); %prealloc for speed
allDVs = nan(nTestCombs, nInstancesPerTest, nTestPerms); %prealloc for speed

weightsFeaturesByPerms = nan([nFs nTestPerms]);

tic;
for permN = 1:nTestPerms
    shouldPrintVerboseOutput = verbose && (permN==1 || mod(permN, floor(nTestPerms/4)) ==0 );
    if shouldPrintVerboseOutput  %ceil, incase nTestPerms < 10
        disp(['testing permutation ' num2str(permN) '/' num2str(nTestPerms)])
    end
    
    % the first cycle is the test case, thus no shuffling
    if permN == 1
        thisPermsLabels = labels;
    else
        thisPermsLabels = Shuffle(labels);
    end
    
    
    for testCombN = 1:nTestCombs
        testInds = testCombs(testCombN,:);
        trainInds = setdiff(instanceInds,testInds);
        
        thisCombsTestData = data(testInds,:);
        thisCombsTestLabels = thisPermsLabels(testInds,:);
        thisCombsTrainData = data(trainInds,:);
        thisCombsTrainLabels = thisPermsLabels(trainInds,:);
        
        accModel = train(thisCombsTrainLabels, thisCombsTrainData, trainOpts);
        [preds, accPreds, dvs] = predict(thisCombsTestLabels, thisCombsTestData, accModel, predictOpt);
        
        allTestLabels(testCombN,:,permN) = thisCombsTestLabels';
        allPreds(testCombN,:,permN) = preds';
        allDVs(testCombN,:,permN) = dvs';
        
        % left here to double-check my sanity
        accLabelByCombByPerm(testCombN,1,permN) = accPreds(1);
        
    end
    
    %% Significance: train on ALL the data
    weightsModel = train(thisPermsLabels,data,trainOpts);
    weightsFeaturesByPerms(:,permN) = weightsModel.w';
    
%     if shouldPrintVerboseOutput && nTestPerms > 1 && permN ~= nTestPerms
%         sprintf('estimated time left (min): %1.2g',toc * (nTestPerms-permN)/permN/60 )
%     end
    
end
toc

%% Calc Accuracies
combLabelPermAccs = allTestLabels == allPreds; 

overallAccs = mean(mean(combLabelPermAccs,2),1); % collapse across labels & combs
output.acc = overallAccs(1);
output.accNull = mean(overallAccs(2:end));
output.accNullDist = squeeze(overallAccs(2:end));
output.accP = sum(output.accNullDist >= output.acc) / numel(output.accNullDist);

s1CombLabelPermAccs = +combLabelPermAccs; % annoyingly, convert back from logical
s1CombLabelPermAccs(allTestLabels ~= set1Label) = NaN; % assign nan to cross-validation against the other set
s1accs = nanmean(nanmean(s1CombLabelPermAccs,2),1); % collapse across labels & combs
output.s1.acc = s1accs(1);
output.s1.accNull = mean(s1accs(2:end));
output.s1.accNullDist = squeeze(s1accs(2:end));
output.s1.accP = sum(output.s1.accNullDist >= output.s1.acc) / numel(output.s1.accNullDist);

s2CombLabelPermAccs = +combLabelPermAccs; % annoyingly, convert back from logical
s2CombLabelPermAccs(allTestLabels ~= set2Label) = NaN; % assign nan to cross-validation against the other set
s2accs = nanmean(nanmean(s2CombLabelPermAccs,2),1); % collapse across labels & combs
output.s2.acc = s2accs(1);
output.s2.accNull = mean(s2accs(2:end));
output.s2.accNullDist = squeeze(s2accs(2:end));
output.s2.accP = sum(output.s2.accNullDist >= output.s2.acc) / numel(output.s2.accNullDist);


% % % sanity check
% permAcc = mean(accLabelByCombByPerm,1); % collapse across combinations
% acc = permAcc(1);
% accNullDist = permAcc(2:end);
% accNull = mean(accNullDist);
% accP = sum(accNullDist >= acc) / numel(accNullDist);



%% Calc Significance
weightsAsVec = weightsFeaturesByPerms(:,1);
weights = nan(featureDims);
weights(noNanVox) = weightsAsVec;

weightsNullDist = weightsFeaturesByPerms(:,2:end);

% rep to match null size, then compare each element pair to construct p vals below
reppingWeights = repmat(weightsAsVec,[1 nTestPerms-1]); % HAHA get it?? -1 b/c not including unshuffled perm, obvi

% find minimum p val for each feature, then double since 2 tailed
notFdrP = 2* min(mean(reppingWeights >= weightsNullDist,2)...
    ,            mean(reppingWeights <= weightsNullDist,2) ); % 2*min(>=,<=)

% norm vals > 1
notFdrP(notFdrP > 1) = 1;

% I'm just very uneasy with 0s for pvalues
notFdrP(notFdrP==0) = .000000001;

%fdr correct
[~, ~, p] = fdr_bh(notFdrP);


% 
% sigButWeightsWas0 = find(h & weights==0)

% weight0 = find(weights==0);
% sum(weight0)
% 
% nonSigWeightInds = weights ~= 0 & ~h;
% sum(nonSigWeightInds)   

output.weights = weights;
extraOutput.weightsNullDist = weightsNullDist;

output.weightsP = nan(featureDims);
output.weightsP(noNanVox) = p;





output.testLabels = allTestLabels(:,:,1); % only for non-null
output.preds = allPreds(:,:,1);
output.DVs = allDVs(:,:,1);

return

function y = nanmean(x,dim)
% FORMAT: Y = NANMEAN(X,DIM)
% 
%    Average or mean value ignoring NaNs
%
%    This function enhances the functionality of NANMEAN as distributed in
%    the MATLAB Statistics Toolbox and is meant as a replacement (hence the
%    identical name).  
%
%    NANMEAN(X,DIM) calculates the mean along any dimension of the N-D
%    array X ignoring NaNs.  If DIM is omitted NANMEAN averages along the
%    first non-singleton dimension of X.
%
%    Similar replacements exist for NANSTD, NANMEDIAN, NANMIN, NANMAX, and
%    NANSUM which are all part of the NaN-suite.
%
%    See also MEAN

% -------------------------------------------------------------------------
%    author:      Jan Gl�scher
%    affiliation: Neuroimage Nord, University of Hamburg, Germany
%    email:       glaescher@uke.uni-hamburg.de
%    
%    $Revision: 1.1 $ $Date: 2004/07/15 22:42:13 $

if isempty(x)
	y = NaN;
	return
end

if nargin < 2
	dim = min(find(size(x)~=1));
	if isempty(dim)
		dim = 1;
	end
end

% Replace NaNs with zeros.
nans = isnan(x);
x(isnan(x)) = 0; 

% denominator
count = size(x,dim) - sum(nans,dim);

% Protect against a  all NaNs in one dimension
i = find(count==0);
count(i) = ones(size(i));

y = sum(x,dim)./count;
y(i) = i + NaN;



% $Id: nanmean.m,v 1.1 2004/07/15 22:42:13 glaescher Exp glaescher $
